@@ -122,17 +122,6 @@ class Include(Node):
                                    fmt % self.filename)
 
 
-class Goto(Node):
-
-    def __init__(self, start, end, label):
-        Node.__init__(self, start, end)
-        self.label = label
-
-    def __str__(self):
-        return self._string_helper(self.__class__.__name__,
-                                   unicode(self.label))
-
-
 class Expr(Node):
 
     def __init__(self, start, end, expr):
@@ -143,14 +132,6 @@ class Expr(Node):
         return self._string_helper(self.__class__.__name__, unicode(self.expr))
 
 
-class Return(Expr):
-    pass
-
-
-class Delete(Expr):
-    pass
-
-
 class Friend(Expr):
 
     def __init__(self, start, end, expr, namespace):
@@ -287,7 +268,8 @@ class Class(_GenericDeclaration):
     def __str__(self):
         name = self.name
         if self.templated_types:
-            name += '<%s>' % self.templated_types
+            types = ','.join([t for t in self.templated_types])
+            name += '<%s>' % types
         suffix = '%s, %s, %s' % (name, self.bases, self.body)
         return self._type_string_helper(suffix)
 
@@ -315,8 +297,12 @@ class Function(_GenericDeclaration):
         return self.body is not None
 
     def is_exportable(self):
-        if self.return_type and 'static' in self.return_type.modifiers:
-            return False
+        if self.return_type:
+            if (
+                'static' in self.return_type.modifiers or
+                'constexpr' in self.return_type.modifiers
+            ):
+                return False
         return None not in self.namespace
 
     def __str__(self):
@@ -397,18 +383,13 @@ class TypeConverter(object):
     def _get_template_end(self, tokens, start):
         count = 1
         end = start
-        while True:
-            try:
-                token = tokens[end]
-            except IndexError:
-                raise ParseError(tokens)
+        while count:
+            token = tokens[end]
             end += 1
             if token.name == '<':
                 count += 1
             elif token.name == '>':
                 count -= 1
-                if count == 0:
-                    break
         return tokens[start:end - 1], end
 
     def to_type(self, tokens):
@@ -424,8 +405,11 @@ class TypeConverter(object):
         result = []
         name_tokens = []
         reference = pointer = array = False
+        inside_array = False
+        empty_array = True
+        templated_tokens = []
 
-        def add_type(templated_types):
+        def add_type():
             if not name_tokens:
                 return
 
@@ -439,51 +423,43 @@ class TypeConverter(object):
                     names.append(t.name)
             name = ''.join(names)
 
+            templated_types = self.to_type(templated_tokens)
             result.append(Type(name_tokens[0].start, name_tokens[-1].end,
                                name, templated_types, modifiers,
                                reference, pointer, array))
             del name_tokens[:]
+            del templated_tokens[:]
 
         i = 0
         end = len(tokens)
         while i < end:
             token = tokens[i]
-            if token.name == '<':
-                new_tokens, new_end = self._get_template_end(tokens, i + 1)
-                if new_end < end:
-                    if tokens[new_end].name == '::':
-                        name_tokens.append(tokens[new_end])
-                        name_tokens.append(tokens[new_end + 1])
-                        new_end += 2
-                    elif tokens[new_end].name == '*':
-                        pointer = True
-                        new_end += 1
-                    elif tokens[new_end].name == '&':
-                        reference = True
-                        new_end += 1
-                add_type(self.to_type(new_tokens))
-                # If there is a comma after the template, we need to consume
-                # that here otherwise it becomes part of the name.
-                i = new_end
-                reference = pointer = array = False
+            if token.name == ']':
+                inside_array = False
+                if empty_array:
+                    pointer = True
+                else:
+                    array = True
+            elif inside_array:
+                empty_array = False
+            elif token.name == '<':
+                templated_tokens, i = self._get_template_end(tokens, i + 1)
+                continue
             elif token.name == ',' or token.name == '(':
-                add_type([])
+                add_type()
                 reference = pointer = array = False
+                empty_array = True
             elif token.name == '*':
                 pointer = True
             elif token.name == '&':
                 reference = True
             elif token.name == '[':
-                pointer = True
-            elif token.name == ']' or token.name == ')':
-                pass
-            else:
+                inside_array = True
+            elif token.name != ')':
                 name_tokens.append(token)
             i += 1
 
-        if name_tokens:
-            # No '<' in the tokens, just a simple name and no template.
-            add_type([])
+        add_type()
         return result
 
     def declaration_to_parts(self, parts, needs_name_removed):
@@ -655,9 +631,6 @@ class ASTBuilder(object):
 
         self.tokens = token_stream
         self.filename = filename
-        # TODO(nnorwitz): use a better data structure (deque) for the queue.
-        # Switching directions of the "queue" improved perf by about 25%.
-        # Using a deque should be even better since we access from both sides.
         self.token_queue = []
         self.namespace_stack = namespace_stack[:]
         self.define = set()
@@ -680,8 +653,9 @@ class ASTBuilder(object):
 
     def generate(self):
         while True:
-            token = self._get_next_token()
-            if not token:
+            try:
+                token = self._get_next_token()
+            except StopIteration:
                 break
 
             # Dispatch on the next token type.
@@ -692,13 +666,9 @@ class ASTBuilder(object):
                 self.namespace_stack.pop()
                 continue
 
-            try:
-                result = self._generate_one(token)
-                if result is not None:
-                    yield result
-            except:
-                self.handle_error('exception', token)
-                raise
+            result = self._generate_one(token)
+            if result is not None:
+                yield result
 
     def _create_variable(self, pos_token, name, type_name, type_modifiers,
                          ref_pointer_name_seq, templated_types=None, value=''):
@@ -1065,6 +1035,10 @@ class ASTBuilder(object):
         if token.name == '&' or token.name == '&&':
             token = self._get_next_token()
 
+        # Handle trailing return types.
+        if token.name == '->':
+            return_type, token = self._get_var_tokens_up_to(False, '{', ';')
+
         if token.name == '}' or token.token_type == tokenize.PREPROCESSOR:
             self._add_back_token(token)
             token = tokenize.Token(tokenize.SYNTAX, ';', 0, 0)
@@ -1176,39 +1150,6 @@ class ASTBuilder(object):
         class_name = names[-1]
         return return_type, class_name
 
-    def handle_bool(self):
-        pass
-
-    def handle_char(self):
-        pass
-
-    def handle_int(self):
-        pass
-
-    def handle_long(self):
-        pass
-
-    def handle_short(self):
-        pass
-
-    def handle_double(self):
-        pass
-
-    def handle_float(self):
-        pass
-
-    def handle_void(self):
-        pass
-
-    def handle_wchar_t(self):
-        pass
-
-    def handle_unsigned(self):
-        pass
-
-    def handle_signed(self):
-        pass
-
     def _get_nested_type(self, ctor):
         # Handle strongly typed enumerations.
         token = self._get_next_token()
@@ -1323,12 +1264,6 @@ class ASTBuilder(object):
     def handle_enum(self):
         return self._get_nested_type(Enum)
 
-    def handle_auto(self):
-        pass
-
-    def handle_register(self):
-        pass
-
     def handle_const(self):
         self._handling_const = True
         token = self._get_next_token()
@@ -1342,42 +1277,28 @@ class ASTBuilder(object):
     def handle_extern(self):
         pass
 
-    def handle_static(self):
-        pass
-
     def handle_virtual(self):
         # What follows must be a method.
-        token = token2 = self._get_next_token()
+        token = self._get_next_token()
         if token.name == 'inline':
-            # HACK(nnorwitz): handle inline dtors by ignoring 'inline'.
-            token2 = self._get_next_token()
-        if token2.token_type == tokenize.SYNTAX and token2.name == '~':
+            token = self._get_next_token()
+        if token.token_type == tokenize.SYNTAX and token.name == '~':
             return self.get_method(FUNCTION_VIRTUAL + FUNCTION_DTOR, None)
-        assert_parse(token.token_type == tokenize.NAME or token.name == '::',
-                     token)
         return_type_and_name = self._get_tokens_up_to('(')
         return_type_and_name.insert(0, token)
-        if token2 is not token:
-            return_type_and_name.insert(1, token2)
         return self._get_method(return_type_and_name, FUNCTION_VIRTUAL,
                                 None, False)
 
-    def handle_volatile(self):
-        pass
-
-    def handle_mutable(self):
-        pass
-
     def handle_public(self):
         assert_parse(self.in_class, 'expected to be in a class')
         self.visibility = VISIBILITY_PUBLIC
 
     def handle_protected(self):
-        assert self.in_class
+        assert_parse(self.in_class, 'expected to be in a class')
         self.visibility = VISIBILITY_PROTECTED
 
     def handle_private(self):
-        assert self.in_class
+        assert_parse(self.in_class, 'expected to be in a class')
         self.visibility = VISIBILITY_PRIVATE
 
     def handle_friend(self):
@@ -1397,26 +1318,6 @@ class ASTBuilder(object):
         assert result
         return Friend(result.start, result.end, result, self.namespace_stack)
 
-    def handle_static_cast(self):
-        pass
-
-    def handle_const_cast(self):
-        pass
-
-    def handle_dynamic_cast(self):
-        pass
-
-    def handle_reinterpret_cast(self):
-        pass
-
-    def handle_new(self):
-        pass
-
-    def handle_delete(self):
-        tokens = self._get_tokens_up_to(';')
-        assert tokens
-        return Delete(tokens[0].start, tokens[0].end, tokens)
-
     def handle_typedef(self):
         token = self._get_next_token()
         if (token.token_type == tokenize.NAME and
@@ -1431,29 +1332,33 @@ class ASTBuilder(object):
         # Get the remainder of the typedef up to the semi-colon.
         tokens.extend(self._get_tokens_up_to(';'))
 
-        # TODO(nnorwitz): clean all this up.
-        assert tokens
         name = tokens.pop()
-        indices = name
-        if tokens:
-            indices = tokens[0]
-        if not indices:
-            indices = token
         if name.name == ')':
-            # HACK(nnorwitz): Handle pointers to functions "properly".
-            if (len(tokens) >= 4 and
-                    tokens[1].name == '(' and tokens[2].name == '*'):
-                tokens.append(name)
-                name = tokens[3]
-        elif name.name == ']':
-            # HACK(nnorwitz): Handle arrays properly.
-            if len(tokens) >= 2:
-                tokens.append(name)
-                name = tokens[1]
+            tokens.append(name)
+            end = len(tokens) - 2
+            count = 1
+            while count:
+                if tokens[end].name == '(':
+                    count -= 1
+                elif tokens[end].name == ')':
+                    count += 1
+                end -= 1
+            start = end
+            if tokens[start].name == ')':
+                name = tokens[start - 1]
+                while tokens[start].name != '(':
+                    start -= 1
+            else:
+                name = tokens[start]
+            del tokens[start:end + 1]
+        elif name.name == ']' and len(tokens) >= 2:
+            tokens.append(name)
+            name = tokens[1]
+            del tokens[1]
         new_type = tokens
         if tokens and isinstance(tokens[0], tokenize.Token):
-            new_type = self.converter.to_type(tokens)[0]
-        return Typedef(indices.start, indices.end, name.name,
+            new_type = self.converter.to_type(tokens)
+        return Typedef(name.start, name.end, name.name,
                        new_type, self.namespace_stack)
 
     def handle_typeid(self):
@@ -1491,12 +1396,12 @@ class ASTBuilder(object):
     def handle_template(self):
         token = self._get_next_token()
 
-        assert_parse(token.token_type == tokenize.SYNTAX, token)
-        assert_parse(token.name == '<', token)
+        templated_types = None
+        if token.token_type == tokenize.SYNTAX and token.name == '<':
+            templated_types = self._get_templated_types()
+            # TODO(nnorwitz): for now, just ignore the template params.
+            token = self._get_next_token()
 
-        templated_types = self._get_templated_types()
-        # TODO(nnorwitz): for now, just ignore the template params.
-        token = self._get_next_token()
         if token.token_type == tokenize.NAME:
             if token.name == 'class':
                 return self._get_class(Class,
@@ -1633,7 +1538,7 @@ class ASTBuilder(object):
             self._add_back_token(token)
 
         return class_type(class_token.start, class_token.end, class_name,
-                          bases, None, body, self.namespace_stack)
+                          bases, templated_types, body, self.namespace_stack)
 
     def handle_namespace(self):
         token = self._get_next_token()
@@ -1672,66 +1577,10 @@ class ASTBuilder(object):
         # This must be a ctor.
         return self.get_method(FUNCTION_CTOR, None)
 
-    def handle_this(self):
-        pass  # Nothing to do.
-
     def handle_operator(self):
         # Pull off the next token(s?) and make that part of the method name.
         pass
 
-    def handle_sizeof(self):
-        pass
-
-    def handle_case(self):
-        pass
-
-    def handle_switch(self):
-        pass
-
-    def handle_default(self):
-        token = self._get_next_token()
-        assert token.token_type == tokenize.SYNTAX
-        assert token.name == ':'
-
-    def handle_if(self):
-        pass
-
-    def handle_else(self):
-        pass
-
-    def handle_return(self):
-        tokens = self._get_tokens_up_to(';')
-        return Return(tokens[0].start, tokens[0].end, tokens)
-
-    def handle_goto(self):
-        tokens = self._get_tokens_up_to(';')
-        assert_parse(len(tokens) == 1, tokens)
-        return Goto(tokens[0].start, tokens[0].end, tokens[0].name)
-
-    def handle_try(self):
-        pass  # Not needed yet.
-
-    def handle_catch(self):
-        pass  # Not needed yet.
-
-    def handle_throw(self):
-        pass  # Not needed yet.
-
-    def handle_while(self):
-        pass
-
-    def handle_do(self):
-        pass
-
-    def handle_for(self):
-        pass
-
-    def handle_break(self):
-        self._ignore_up_to(';')
-
-    def handle_continue(self):
-        self._ignore_up_to(';')
-
 
 def builder_from_source(source, filename, quiet=False):
     """Utility method that returns an ASTBuilder from source code.
